@@ -14,19 +14,21 @@ $VERSION = '0.02';
 
 # uses Calc, but only features the strictly necc. methods.
 
-use Math::BigInt::Calc v0.17;
+use Math::BigInt::Calc '0.33';
 
 BEGIN
   {
+  no strict 'refs';
   foreach (qw/	base_len new zero one two copy str num add sub mul div inc dec
 		acmp len digit zeros
 		is_zero is_one is_odd is_even is_one check
+		to_small to_large
 		/)
     {
     my $name  = "Math::BigInt::Calc::_$_";
-    no strict 'refs';
     *{"Math::BigInt::BareCalc::_$_"} = \&$name;
     }
+  print "# BareCalc using Calc v$Math::BigInt::Calc::VERSION\n";
   }
 
 # catch and throw away
